<?php

declare(strict_types = 1);

namespace Drupal\commerce_otpsp;

use Cheppers\OtpspClient\Checksum;
use Cheppers\OtpspClient\DataType\Redirect;
use Cheppers\OtpspClient\OtpSimplePayClient;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

class PaymentDataProvider implements ContainerInjectionInterface {

  /**
   * @var \Drupal\commerce_payment\Entity\PaymentInterface
   */
  protected $payment;

  /**
   * @var \Drupal\commerce_order\Entity\OrderInterface
   */
  protected $order;

  /**
   * @var array
   */
  protected $configuration;

  /**
   * @var \Drupal\commerce_store\Entity\StoreInterface
   */
  protected $store;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * @var \Cheppers\OtpspClient\OtpSimplePayClient
   */
  protected $client;

  /**
   * @var \Cheppers\OtpspClient\Checksum
   */
  protected $checksum;

  /**
   * @var \Drupal\commerce_shipping\Entity\ShipmentInterface
   */
  protected $shipment;

  /**
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * @var int
   */
  protected $discount = 0;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $container->get('request_stack'),
      $container->get('current_user'),
      $container->get('language_manager'),
      $container->get('date.formatter'),
      $container->get('commerce_otpsp.client'),
      $container->get('commerce_otpsp.checksum')
    );
  }

  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    ModuleHandlerInterface $moduleHandler,
    RequestStack $requestStack,
    AccountProxyInterface $currentUser,
    LanguageManagerInterface $languageManager,
    DateFormatter $dateFormatter,
    OtpSimplePayClient $client,
    Checksum $checksum
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->client = $client;
    $this->checksum = $checksum;
    $this->requestStack = $requestStack;
    $this->currentUser = $currentUser;
    $this->languageManager = $languageManager;
    $this->dateFormatter = $dateFormatter;
    $this->moduleHandler = $moduleHandler;
  }

  public function setUp($paymentId) {
    /** @var \Drupal\commerce_payment\PaymentStorageInterface $paymentStorage */
    $paymentStorage = $this->entityTypeManager->getStorage('commerce_payment');

    /** @var \Drupal\commerce_order\OrderStorage $orderStorage */
    $orderStorage = $this->entityTypeManager->getStorage('commerce_order');

    $this->payment = $paymentStorage->load($paymentId);
    $this->order = $orderStorage->load($this->payment->getOrderId());
    $this->configuration = $this->payment->getPaymentGateway()->getPluginConfiguration();
    $this
      ->client
      ->setMerchantId($this->configuration['merchant_id'])
      ->setSecretKey($this->configuration['secret_key'])
      ->setBaseUri($this->getUrlByMode($this->configuration['mode']));
  }

  public function getPreparedData(): array {
    $routeParameters = [
      'commerce_order' => $this->order->id(),
      'step' => 'payment',
    ];
    $urlOptions = [
      'absolute' => TRUE,
      'query' => [
        'REFNOEXT' => $this->payment->uuid(),
      ],
    ];

    $data = [
      'merchantId' => $this->configuration['merchant_id'],
      'customerEmail' => $this->order->getEmail(),
      'timeoutUrl' => Url::fromRoute(
        'commerce_payment.checkout.cancel',
        $routeParameters,
        $urlOptions
      )->toString(),
      'backrefUrl' => Url::fromRoute(
        'commerce_payment.checkout.return',
        $routeParameters,
        $urlOptions
      )->toString(),
      'langCode' => $this->getLangCode(),
      'products' => $this->getProductsData(),
      'order' => [
        'paymentId' => $this->payment->uuid(),
        'orderDate' => $this->dateFormatter->format((int) $this->order->getChangedTime(), 'custom', 'Y-m-d H:i:s'),
        'currency' => $this->payment->getAmount()->getCurrencyCode(),
        'shippingPrice' => $this->configuration['send_shipping'] ? (string) $this->getShipmentPrice() : '0',
        'discount' => $this->discount !== 0 ? (string) $this->discount : '0',
      ],
    ];

    if ($this->configuration['send_billing']) {
      $data['billingAddress'] = $this->getBillingData();
    }

    if ($this->configuration['send_shipping']) {
      $data['shippingAddress'] = $this->getShippingData();
    }

    $redirect = Redirect::__set_state($data);

    $this->moduleHandler->alter('commerce_otpsp_redirect', $this->payment, $redirect);

    $postingData = $redirect->exportData();
    $hashValues = $redirect->getHashValues($postingData);

    $postingData[] = [
      'ORDER_HASH' => $this->checksum->calculate($hashValues, $this->configuration['secret_key']),
    ];

    return $postingData;
  }

  public function getProductsData(): array {
    $products = [];
    $items = $this->order->getItems();

    $i = 0;
    foreach ($items as $item) {
      $products[] = [
        'productName' => $item->getTitle(),
        'sku' => $item->id(),
        'price' => $item->getUnitPrice()->getNumber(),
        'quantity' => $item->getQuantity(),
        'vat' => '0',
      ];
      $this->discount = $this->getDiscountAmount($item->getAdjustments(['promotion']));
      $i++;
    }

    return $products;
  }

  public function getBillingData(): array {
    $address = $this
      ->order
      ->getBillingProfile()
      ->get('address');

    if ($address->isEmpty()) {
      return [];
    }

    return $this->transformAddressField($address->first()->getValue());
  }

  public function getShippingData(): array {
    if (!$this->entityTypeManager->hasDefinition('commerce_shipment')) {
      return [];
    }

    /** @var \Drupal\commerce_shipping\ShipmentStorageInterface $shipmentStorage */
    $shipmentStorage = $this->entityTypeManager->getStorage('commerce_shipment');

    $shipments = $shipmentStorage->loadMultipleByOrder($this->order);

    /** @var \Drupal\commerce_shipping\Entity\ShipmentInterface $shipment */
    $shipment = reset($shipments);
    if (!$shipment) {
      return [];
    }

    $address = $shipment
      ->getShippingProfile()
      ->get('address');

    if ($address->isEmpty()) {
      return [];
    }

    return $this->transformAddressField($address->first()->getValue());
  }

  public function transformAddressField(array $addressValue): array {
    return [
      'firstName'    => $addressValue['given_name'],
      'lastName'     => $addressValue['family_name'],
      'countryCode'  => $addressValue['country_code'],
      'city'         => $addressValue['locality'],
      'addressLine'  => $addressValue['address_line1'],
      'addressLine2' => $addressValue['address_line2'],
      'postalCode'   => $addressValue['postal_code'],
    ];
  }

  public function getUrlByMode(string $mode): string {
    $host = ($mode === 'live') ? 'secure' : 'sandbox';

    return "https://$host.simplepay.hu/payment";
  }

  public function getShipmentPrice(): float {
    /** @var \Drupal\commerce_shipping\ShipmentStorageInterface $shipmentStorage */
    $shipmentStorage = $this->entityTypeManager->getStorage('commerce_shipment');
    /** @var \Drupal\commerce_shipping\Entity\ShipmentInterface[] $shipment */
    $shipments = $shipmentStorage->loadMultipleByOrder($this->order);

    $shippingPrice = 0;

    /** @var \Drupal\commerce_shipping\Entity\ShipmentInterface $shipment */
    foreach ($shipments as $shipment) {
      $shippingPrice += (float) $shipment->getAmount()->getNumber();
    }

    return $shippingPrice;
  }

  protected function getLangCode(): string {
    $supportedLangCodes = $this->client->getSupportedLanguages();

    $langCode = $this->languageManager->getCurrentLanguage()->getId();
    if (in_array($langCode, $supportedLangCodes)) {
      return $langCode;
    }

    $langCode = $this->currentUser->getPreferredLangcode(FALSE);
    if ($langCode && in_array($langCode, $supportedLangCodes)) {
      return $langCode;
    }

    $langCode = $this->languageManager->getDefaultLanguage()->getId();

    return in_array($langCode, $supportedLangCodes) ? $langCode : 'HU';
  }

  protected function getDiscountAmount(array $adjustments): int {
    if (!$adjustments) {
      return $this->discount;
    }

    $discount = 0;

    /** @var \Drupal\commerce_order\Adjustment $adjustment */
    foreach ($adjustments as $adjustment) {
      $discount += (int) $adjustment->getAmount()->getNumber();
    }

    return $this->discount + abs($discount);
  }

}
