<?php

declare(strict_types = 1);

namespace Drupal\commerce_otpsp\Plugin\Commerce\PaymentGateway;

use Cheppers\OtpspClient\OtpSimplePayClientInterface;
use Cheppers\OtpspClient\UrlParser;
use Cheppers\OtpspClient\Utils;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_payment\Entity\Payment;
use Exception;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Provides SimplePay payment gateway.
 *
 * @\Drupal\commerce_payment\Annotation\CommercePaymentGateway(
 *   id = "commerce_otpsp_redirect",
 *   label = @\Drupal\Core\Annotation\Translation("SimplePay (Redirect to SimplePay)"),
 *   display_label = @\Drupal\Core\Annotation\Translation("SimplePay by OTP Mobile"),
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_otpsp\PluginForm\SimplePayRedirectOffSitePaymentForm",
 *   },
 *   credit_card_types = {
 *     "mastercard",
 *     "maestro",
 *     "visa",
 *     "amex",
 *   },
 * )
 */
class SimplePay extends OffsitePaymentGatewayBase {

  /**
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * @var \Cheppers\OtpspClient\OtpSimplePayClientInterface
   */
  protected $client;

  /**
   * @var \Cheppers\OtpspClient\UrlParser
   */
  protected $urlParser;

  /**
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.commerce_payment_type'),
      $container->get('plugin.manager.commerce_payment_method_type'),
      $container->get('datetime.time'),
      $container->get('module_handler'),
      $container->get('commerce_otpsp.client'),
      $container->get('commerce_otpsp.url_parser'),
      $container->get('database')
    );
  }

  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    PaymentTypeManager $payment_type_manager,
    PaymentMethodTypeManager $payment_method_type_manager,
    TimeInterface $time,
    ModuleHandlerInterface $moduleHandler,
    OtpSimplePayClientInterface $client,
    UrlParser $urlParser,
    Connection $database
  ) {
    $this->moduleHandler = $moduleHandler;
    $this->client = $client;
    $this->urlParser = $urlParser;
    $this->database = $database;
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $entity_type_manager,
      $payment_type_manager,
      $payment_method_type_manager,
      $time
    );
    $this
      ->client
      ->setMerchantId($this->configuration['merchant_id'])
      ->setSecretKey($this->configuration['secret_key']);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'merchant_id' => '',
      'secret_key' => '',
      'send_billing' => FALSE,
      'send_shipping' => FALSE,
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['merchant_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Merchant ID'),
      '#description' => $this->t('This is the merchant ID from OTPSimple.'),
      '#default_value' => $this->configuration['merchant_id'],
      '#required' => TRUE,
    ];

    $form['secret_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Secret key'),
      '#description' => $this->t('The secret key from your OTPSimple account'),
      '#default_value' => $this->configuration['secret_key'],
      '#required' => TRUE,
    ];

    $form['send_billing'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Send billing'),
      '#description' => $this->t('Send checkout billing info to Simple'),
      '#default_value' => $this->configuration['send_billing'],
      '#required' => FALSE,
    ];

    $form['send_shipping'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Send shipping'),
      '#description' => $this->t('Send checkout shipping info to Simple'),
      '#default_value' => $this->configuration['send_shipping'],
      '#required' => FALSE,
      '#access' => $this->moduleHandler->moduleExists('commerce_shipping'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $values = $form_state->getValue($form['#parents']);
    $this->configuration['merchant_id'] = $values['merchant_id'];
    $this->configuration['secret_key'] = $values['secret_key'];
    $this->configuration['send_billing'] = $values['send_billing'];
    $this->configuration['send_shipping'] = $values['send_shipping'];
  }

  /**
   * {@inheritdoc}
   */
  public function onNotify(Request $request) {
    $ipnData = [];
    parse_str($request->getContent(), $ipnData);

    if (!$ipnData || !$this->client->isValidChecksum($ipnData['HASH'], Utils::flatArray($ipnData, ['HASH']))) {
      // @todo Log - Check you Merchant ID and your secret key configured in
      // the Payment gateway instance.
      throw new PaymentGatewayException('Invalid instant payment notification.');
    }

    $ipn = $this->client->parseInstantPaymentNotificationRequest($request->getContent());

    $ipnResponse = $this->client->getInstantPaymentNotificationFailedResponse();
    $failedResponse = new Response(
      $ipnResponse['body'],
      $ipnResponse['statusCode'],
      $ipnResponse['headers']
    );

    $ipnResponse = $this->client->getInstantPaymentNotificationSuccessResponse($ipn);
    $successResponse = new Response(
      $ipnResponse['body'],
      $ipnResponse['statusCode'],
      $ipnResponse['headers']
    );

    $payment = $this->getPaymentByUuid($ipn->refNoExt);
    if (!$payment) {
      // Payment with the given UUID is not exists any more.
      // We don't want to receive the same IPN again.
      return $successResponse;
    }

    try {
      $payment
        ->set('state', 'completed')
        ->set('remote_id', $ipn->refNo)
        ->set('remote_state', $ipn->orderStatus)
        ->save();
    }
    catch (Exception $exception) {
      return $failedResponse;
    }

    return $successResponse;
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $backrefData = $this->client->parseBackRefRequest($request->getUri());

    $url = $request->getSchemeAndHttpHost()
      . $request->getBaseUrl()
      . $request->getRequestUri();
    $urlWithoutCtrl = $this->urlParser->removeQueryVariable(urldecode($url), $this->client::CONTROL_KEY);

    if (!$this->client->isValidChecksum($backrefData->ctrl, [$urlWithoutCtrl])) {
      throw new PaymentGatewayException('Invalid onReturn request.');
    }

    $payment = $this->getPaymentByUuid($backrefData->refNoExt);
    if (!$payment) {
      throw new PaymentGatewayException('Payment not found');
    }

    if (!$this->client->isPaymentSuccess($backrefData->returnCode)) {
      $payment
        ->set('remote_id', $backrefData->payRefNo)
        ->setState('authorization_voided')
        ->save();

      throw new PaymentGatewayException('Payment failed');
    }

    if ($payment->getState()->getId() === 'new') {
      $payment
        ->set('remote_id', $backrefData->payRefNo)
        ->setState('authorization')
        ->save();

      return;
    }

    if ($payment->getRemoteId() !== $backrefData->payRefNo) {
      $payment
        ->set('remote_id', $backrefData->payRefNo)
        ->save();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onCancel(OrderInterface $order, Request $request) {
    $backrefData = $this->client->parseBackRefRequest($request->getUri());

    $payment = $this->getPaymentByUuid($backrefData->refNoExt);
    if (!$payment) {
      return;
    }

    $payment
      ->setState('authorization_voided')
      ->save();

    $this
      ->messenger()
      ->addWarning(
        $this->t(
          'You have canceled checkout at @gateway but may resume the checkout process here when you are ready.',
          [
            '@gateway' => $this->getDisplayLabel(),
          ]
        )
      );
  }

  protected function getPaymentByUuid(string $uuid): ?Payment {
    /** @var \Drupal\commerce_payment\Entity\Payment[] $payments */
    $payments = $this
      ->entityTypeManager
      ->getStorage('commerce_payment')
      ->loadByProperties(['uuid' => $uuid]);

    return reset($payments) ?: NULL;
  }

}
