<?php

declare(strict_types = 1);

namespace Drupal\Tests\commerce_otpsp\FunctionalJavascript;

use Drupal\commerce_payment\Entity\PaymentGateway;
use Drupal\Tests\commerce\FunctionalJavascript\CommerceWebDriverTestBase;

/**
 * Tests the SimplePay.
 *
 * @group commerce_otpsp
 */
class SimplePayTest extends CommerceWebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'commerce_cart',
    'commerce_product',
    'commerce_order',
    'commerce_payment',
    'commerce_store',
    'commerce_price',
    'commerce_checkout',
    'commerce_shipping',
    'commerce_otpsp',
  ];

  /**
   * @var \Drupal\commerce_product\Entity\ProductVariationInterface
   */
  protected $productVariation = NULL;

  /**
   * @var \Drupal\commerce_product\Entity\ProductInterface
   */
  protected $product = NULL;

  /**
   * @var \Drupal\commerce_payment\Entity\PaymentGatewayInterface
   */
  protected $paymentGateway = NULL;

  /**
   * @var string
   */
  protected $cardNumberSuccessVisa = '4908366099900425';

  /**
   * @var string
   */
  protected $cardNumberSuccessMasterCard = '5587402000012011';

  /**
   * @var string
   */
  protected $cardNumberFailed = '4111111111111111';

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    $account = $this->drupalCreateUser();
    $this->drupalLogin($account);

    /** @var \Drupal\commerce_price\Entity\Currency $currency */
    $currency = $this->createEntity('commerce_currency', [
      'currencyCode' => 'HUF',
      'name' => 'Hungarian forint',
      'numericCode' => 348,
      'symbol' => 'HUF',
      'fractionDigits' => 2,
    ]);

    $this->store->setDefaultCurrency($currency);

    /** @var \Drupal\commerce_shipping\Entity\PackageType $packageType */
    $packageType = $this->createEntity('commerce_package_type', [
      'id' => 'package_type_a',
      'label' => 'Package Type A',
      'dimensions' => [
        'length' => 20,
        'width' => 20,
        'height' => 20,
        'unit' => 'mm',

      ],
      'weight' => [
        'number' => 20,
        'unit' => 'g',
      ],
    ]);
    $this->container->get('plugin.manager.commerce_package_type')->clearCachedDefinitions();

    $this->createEntity('commerce_shipping_method', [
      'name' => 'Overnight shipping',
      'stores' => [$this->store->id()],
      'plugin' => [
        'target_plugin_id' => 'flat_rate',
        'target_plugin_configuration' => [
          'default_package_type' => 'commerce_package_type:' . $packageType->get('uuid'),
          'rate_label' => 'Overnight shipping',
          'rate_amount' => [
            'number' => '100',
            'currency_code' => 'HUF',
          ],
        ],
      ],
    ]);

    $this->paymentGateway = PaymentGateway::create([
      'id' => 'simplepay',
      'label' => 'SimplePay',
      'plugin' => 'commerce_otpsp_redirect',
      'configuration' => [
        'display_label' => 'SimplePay',
        'merchant_id' => 'PUBLICTESTHUF',
        'secret_key' => 'FxDa5w314kLlNseq2sKuVwaqZshZT5d6',
        'send_billing' => '1',
      ],
    ]);
    $this->paymentGateway->save();

    $this->productVariation = $this->createEntity('commerce_product_variation', [
      'type' => 'default',
      'sku' => '0001',
      'price' => [
        'number' => '20',
        'currency_code' => 'HUF',
      ],
      'weight' => [
        'number' => '1',
        'unit' => 'kg',
      ],
    ]);

    $this->product = $this->createEntity('commerce_product', [
      'type' => 'default',
      'title' => 'my product 01',
      'id' => 'my product 01',
      'stores' => [$this->store],
      'variations' => [$this->productVariation],
    ]);
  }

  public function testCancelPayment() {
    $this->drupalGet($this->product->toUrl()->toString());
    $this->submitForm([], 'Add to cart');
    $this->drupalGet('checkout/1');

    $this->submitForm([
      'payment_information[billing_information][address][0][address][given_name]' => 'Johnny',
      'payment_information[billing_information][address][0][address][family_name]' => 'Appleseed',
      'payment_information[billing_information][address][0][address][address_line1]' => '123 New York Drive',
      'payment_information[billing_information][address][0][address][locality]' => 'New York City',
      'payment_information[billing_information][address][0][address][administrative_area]' => 'NY',
      'payment_information[billing_information][address][0][address][postal_code]' => '10001',
    ], 'Continue to review');
    $this->submitForm([], 'Pay and complete purchase');
    $this->getSession()->getPage()->pressButton('cancelPayment');
    $this->getSession()->wait(5000);
    $this->assertSession()->pageTextContains('You have canceled checkout at SimplePay but may resume the checkout process here when you are ready.');
  }

  public function testUnsuccessfulCard() {
    $this->drupalGet($this->product->toUrl()->toString());
    $this->submitForm([], 'Add to cart');
    $this->drupalGet('checkout/1');

    $this->submitForm([
      'payment_information[billing_information][address][0][address][given_name]' => 'Johnny',
      'payment_information[billing_information][address][0][address][family_name]' => 'Appleseed',
      'payment_information[billing_information][address][0][address][address_line1]' => '123 New York Drive',
      'payment_information[billing_information][address][0][address][locality]' => 'New York City',
      'payment_information[billing_information][address][0][address][administrative_area]' => 'NY',
      'payment_information[billing_information][address][0][address][postal_code]' => '10001',
    ], 'Continue to review');
    $this->submitForm([], 'Pay and complete purchase');
    $card = $this->getSession()->getPage()->findById('cardNoSelect');
    $card->setValue($this->cardNumberFailed);
    $this->getSession()->getPage()->pressButton('finishPayment');
    $this->getSession()->wait(5000);
    $this->assertSession()->pageTextContains('Payment failed at the payment server. Please review your information and try again.');
  }

  public function testDataSend() {
    $this->paymentGateway->setPluginConfiguration([
      'display_label' => 'SimplePay',
      'merchant_id' => 'PUBLICTESTHUF',
      'secret_key' => 'FxDa5w314kLlNseq2sKuVwaqZshZT5d6',
      'send_billing' => '1',
      'send_shipping' => '1',
    ]);

    /** @var \Drupal\commerce_product\Entity\ProductVariationType $productVariationType */
    $productVariationType = \Drupal::entityTypeManager()->getStorage('commerce_product_variation_type')->load('default');
    $productVariationType->setTraits(['purchasable_entity_shippable']);
    $productVariationType->save();

    /** @var \Drupal\commerce_order\Entity\OrderType $orderType */
    $orderType = \Drupal::entityTypeManager()->getStorage('commerce_order_type')->load('default');
    $orderType->setThirdPartySetting('commerce_checkout', 'checkout_flow', 'shipping');
    $orderType->setThirdPartySetting('commerce_shipping', 'shipment_type', 'default');
    $orderType->save();

    $fieldDefinition = commerce_shipping_build_shipment_field_definition($orderType->id());
    $this->container->get('commerce.configurable_field_manager')->createField($fieldDefinition);

    $traitManager = $this->container->get('plugin.manager.commerce_entity_trait');
    $trait = $traitManager->createInstance('purchasable_entity_shippable');
    $traitManager->installTrait($trait, 'commerce_product_variation', 'default');

    $this->paymentGateway->save();
    $this->drupalGet($this->product->toUrl()->toString());
    $this->submitForm([], 'Add to cart');
    $this->drupalGet('checkout/1');

    $page = $this->getSession()->getPage();
    $address = [
      'given_name' => 'John',
      'family_name' => 'Smith',
      'address_line1' => '1098 Alta Ave',
      'locality' => 'Mountain View',
      'administrative_area' => 'CA',
      'postal_code' => '94043',
    ];

    $shippingAddressPrefix = 'shipping_information[shipping_profile][address][0][address]';
    $billingAddressPrefix = 'payment_information[billing_information][address][0][address]';

    foreach ($address as $property => $value) {
      $page->fillField($shippingAddressPrefix . '[' . $property . ']', $value);
      $page->fillField($billingAddressPrefix . '[' . $property . ']', $value);
    }

    $page->findButton('Recalculate shipping')->click();
    $this->waitForAjaxToFinish();

    $page->pressButton('Continue to review');
    $this->submitForm([], 'Pay and complete purchase');

    /** @var \Drupal\commerce_order\Entity\Order $order */
    $order = \Drupal::entityTypeManager()->getStorage('commerce_order')->load(1);
    $orderTotalPrice = $order->getTotalPrice();
    $orderName = $this->product->getTitle();

    /** @var \Drupal\commerce_shipping\ShipmentStorageInterface $shipmentStorage */
    $shipmentStorage = \Drupal::entityTypeManager()->getStorage('commerce_shipment');
    $shipments = $shipmentStorage->loadMultipleByOrder($order);

    $shipmentPrice = 0;

    /** @var \Drupal\commerce_shipping\Entity\Shipment $shipment */
    foreach ($shipments as $shipment) {
      $shipmentPrice += (int) $shipment->getAmount()->getNumber();
    }

    $shipmentCurrency = reset($shipments)->getAmount()->getCurrencyCode();

    $page = $this->getSession()->getPage();
    $sumField = $page->findById('sumRow');
    $productsField = $page->findById('termekek');
    $productPrice = $this->productVariation->getPrice();
    $productQuantity = count($order->getItems());

    $this->assertSame("TOTAL: $orderTotalPrice", $sumField->getText());
    $this->assertSame(
      "$productPrice $productQuantity x  $orderName $shipmentPrice $shipmentCurrency Shipping cost :",
      $productsField->getText()
    );

    $nameField = $page->findById('delivery_name');
    $cityField = $page->findById('delivery_city');
    $streetField = $page->findById('delivery_address');
    $countryField = $page->findById('delivery_countryView');
    $countryCode = $countryField->getAttribute('data-code');

    $this->assertSame('John Smith', $nameField->getText());
    $this->assertSame('94043 / Mountain View', $cityField->getText());
    $this->assertSame('1098 Alta Ave', $streetField->getText());
    $this->assertSame('US', $countryCode);
  }

}
